@@ -1,28 +1,22 @@
 import functools
-import sys
 import typing
+import warnings
 
-import anyio
-
-if sys.version_info >= (3, 10):  # pragma: no cover
-    from typing import ParamSpec
-else:  # pragma: no cover
-    from typing_extensions import ParamSpec
-
-try:
-    import contextvars  # Python 3.7+ only or via contextvars backport.
-except ImportError:  # pragma: no cover
-    contextvars = None  # type: ignore
-
+import anyio.to_thread
 
 T = typing.TypeVar("T")
-P = ParamSpec("P")
 
 
-async def run_until_first_complete(*args: typing.Tuple[typing.Callable, dict]) -> None:
+async def run_until_first_complete(*args: typing.Tuple[typing.Callable, dict]) -> None:  # type: ignore[type-arg]  # noqa: E501
+    warnings.warn(
+        "run_until_first_complete is deprecated "
+        "and will be removed in a future version.",
+        DeprecationWarning,
+    )
+
     async with anyio.create_task_group() as task_group:
 
-        async def run(func: typing.Callable[[], typing.Coroutine]) -> None:
+        async def run(func: typing.Callable[[], typing.Coroutine]) -> None:  # type: ignore[type-arg]  # noqa: E501
             await func()
             task_group.cancel_scope.cancel()
 
@@ -30,16 +24,12 @@ async def run_until_first_complete(*args: typing.Tuple[typing.Callable, dict]) -
             task_group.start_soon(run, functools.partial(func, **kwargs))
 
 
+# TODO: We should use `ParamSpec` here, but mypy doesn't support it yet.
+# Check https://github.com/python/mypy/issues/12278 for more details.
 async def run_in_threadpool(
-    func: typing.Callable[P, T], *args: P.args, **kwargs: P.kwargs
+    func: typing.Callable[..., T], *args: typing.Any, **kwargs: typing.Any
 ) -> T:
-    if contextvars is not None:  # pragma: no cover
-        # Ensure we run in the same context
-        child = functools.partial(func, *args, **kwargs)
-        context = contextvars.copy_context()
-        func = context.run  # type: ignore[assignment]
-        args = (child,)  # type: ignore[assignment]
-    elif kwargs:  # pragma: no cover
+    if kwargs:  # pragma: no cover
         # run_sync doesn't accept 'kwargs', so bind them in here
         func = functools.partial(func, **kwargs)
     return await anyio.to_thread.run_sync(func, *args)
